@@ -1,23 +1,23 @@
 # Copyright (C) 2006-2008 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation; either version 2.1 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Lesser General Public License for more details.
 
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """Python bindings for Subversion."""
 
 __author__ = "Jelmer Vernooij <jelmer@samba.org>"
-__version__ = (0, 6, 1)
+__version__ = (0, 6, 8)
 
 NODE_DIR = 2
 NODE_FILE = 1
@@ -60,6 +60,22 @@ ERR_WC_BAD_ADM_LOG = 155009
 ERR_WC_BAD_ADM_LOG_START = 155020
 ERR_WC_NOT_LOCKED = 155005
 ERR_RA_DAV_NOT_VCC = 20014
+ERR_REPOS_HOOK_FAILURE = 165001
+ERR_XML_MALFORMED = 130003
+ERR_MALFORMED_FILE = 200002
+ERR_FS_PATH_SYNTAX = 160005
+ERR_RA_DAV_FORBIDDEN = 175013
+ERR_WC_SCHEDULE_CONFLICT = 155013
+
+AUTH_PARAM_DEFAULT_USERNAME = 'svn:auth:username'
+AUTH_PARAM_DEFAULT_PASSWORD = 'svn:auth:password'
+
+SSL_NOTYETVALID = 0x00000001
+SSL_EXPIRED     = 0x00000002
+SSL_CNMISMATCH  = 0x00000004
+SSL_UNKNOWNCA   = 0x00000008
+SSL_OTHER       = 0x40000000
+
 
 class SubversionException(Exception):
     """A Subversion exception"""
@@ -71,7 +87,10 @@ class SubversionException(Exception):
 
 
 def _check_mtime(m):
-    """Check whether a C extension is out of date."""
+    """Check whether a C extension is out of date.
+    
+    :param m: Python module that is a C extension
+    """
     import os
     (base, _) = os.path.splitext(m.__file__)
     c_file = "%s.c" % base
@@ -85,8 +104,9 @@ try:
     import client, _ra, repos, wc
     for x in client, _ra, repos, wc:
         if not _check_mtime(x):
+            from warnings import warn
             warn("subvertpy extensions are outdated and need to be rebuilt")
             break
-except ImportError:
-    raise ImportError("Unable to load subvertpy extensions - did you build it?")
+except ImportError, e:
+    raise ImportError("Unable to load subvertpy extensions: %s", e.message)
 
