@@ -1,17 +1,18 @@
-/* Copyright © 2008 Jelmer Vernooij <jelmer@samba.org>
+/*
+ * Copyright © 2008-2009 Jelmer Vernooij <jelmer@samba.org>
  * -*- coding: utf-8 -*-
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
@@ -76,6 +77,8 @@ static svn_error_t *py_lock_func (void *baton, const char *path, int do_lock,
 	PyGILState_STATE state = PyGILState_Ensure();
 	if (ra_err != NULL) {
 		py_ra_err = PyErr_NewSubversionException(ra_err);
+	} else {
+		Py_INCREF(py_ra_err);
 	}
 	py_lock = pyify_lock(lock);
 	ret = PyObject_CallFunction((PyObject *)baton, "zbOO", path, do_lock, 
@@ -188,11 +191,18 @@ static PyObject *reporter_abort(PyObject *self)
 }
 
 static PyMethodDef reporter_methods[] = {
-	{ "abort", (PyCFunction)reporter_abort, METH_NOARGS, NULL },
-	{ "finish", (PyCFunction)reporter_finish, METH_NOARGS, NULL },
-	{ "link_path", (PyCFunction)reporter_link_path, METH_VARARGS, NULL },
-	{ "set_path", (PyCFunction)reporter_set_path, METH_VARARGS, NULL },
-	{ "delete_path", (PyCFunction)reporter_delete_path, METH_VARARGS, NULL },
+	{ "abort", (PyCFunction)reporter_abort, METH_NOARGS, 
+		"S.abort()\n"
+		"Abort this report." },
+	{ "finish", (PyCFunction)reporter_finish, METH_NOARGS, 
+		"S.finish()\n"
+		"Finish this report." },
+	{ "link_path", (PyCFunction)reporter_link_path, METH_VARARGS, 
+		"S.link_path(path, url, revision, start_empty, lock_token=None)\n" },
+	{ "set_path", (PyCFunction)reporter_set_path, METH_VARARGS, 
+		"S.set_path(path, revision, start_empty, lock_token=None)\n" },
+	{ "delete_path", (PyCFunction)reporter_delete_path, METH_VARARGS, 
+		"S.delete_path(path)\n" },
 	{ NULL, }
 };
 
@@ -317,6 +327,7 @@ static svn_error_t *py_cb_editor_add_directory(const char *path, void *parent_ba
 	PyObject *self = (PyObject *)parent_baton, *ret;
 	PyGILState_STATE state = PyGILState_Ensure();
 	*child_baton = NULL;
+
 	if (copyfrom_path == NULL) {
 		ret = PyObject_CallMethod(self, "add_directory", "s", path);
 	} else {
@@ -438,15 +449,14 @@ static svn_error_t *py_txdelta_window_handler(svn_txdelta_window_t *window, void
 			py_new_data = PyString_FromStringAndSize(window->new_data->data, window->new_data->len);
 		} else {
 			py_new_data = Py_None;
+			Py_INCREF(py_new_data);
 		}
-		py_window = Py_BuildValue("((LIIiOO))", 
+		py_window = Py_BuildValue("((LIIiNN))", 
 								  window->sview_offset, 
 								  window->sview_len, 
 								  window->tview_len, 
 								  window->src_ops, ops, py_new_data);
 		CB_CHECK_PYRETVAL(py_window);
-		Py_DECREF(ops);
-		Py_DECREF(py_new_data);
 	}
 	ret = PyObject_CallFunction(fn, "O", py_window);
 	Py_DECREF(py_window);
@@ -558,8 +568,10 @@ static svn_error_t *py_file_rev_handler(void *baton, const char *path, svn_revnu
 	Py_DECREF(py_rev_props);
 	CB_CHECK_PYRETVAL(ret);
 
-	*delta_baton = (void *)ret;
-	*delta_handler = py_txdelta_window_handler;
+	if (delta_baton != NULL && delta_handler != NULL) {
+		*delta_baton = (void *)ret;
+		*delta_handler = py_txdelta_window_handler;
+	}
 	PyGILState_Release(state);
 	return NULL;
 }
@@ -637,7 +649,7 @@ static svn_error_t *py_open_tmp_file(apr_file_t **fp, void *callback,
 		const char *path;
 
 		SVN_ERR (svn_io_temp_dir (&path, pool));
-		path = svn_path_join (path, "tempfile", pool);
+		path = svn_path_join (path, "subvertpy", pool);
 		SVN_ERR (svn_io_open_unique_file (fp, NULL, path, ".tmp", TRUE, pool));
 
 		return NULL;
@@ -703,9 +715,10 @@ static void py_progress_func(apr_off_t progress, apr_off_t total, void *baton, a
 
 static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 {
-	char *kwnames[] = { "url", "progress_cb", "auth", "config", "client_string_func", 
-						"open_tmp_file_func", NULL };
-	char *url;
+	char *kwnames[] = { "url", "progress_cb", "auth", "config",
+		                "client_string_func", "open_tmp_file_func", "uuid", 
+						NULL };
+	char *url = NULL, *uuid = NULL;
 	PyObject *progress_cb = Py_None;
 	AuthObject *auth = (AuthObject *)Py_None;
 	PyObject *config = Py_None;
@@ -716,9 +729,10 @@ static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 	svn_auth_baton_t *auth_baton;
 	svn_error_t *err;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|OOOOO", kwnames, &url, 
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|OOOOOz", kwnames, &url, 
 									 &progress_cb, (PyObject **)&auth, &config, 
-									 &client_string_func, &open_tmp_file_func))
+									 &client_string_func, &open_tmp_file_func, 
+									 &uuid))
 		return NULL;
 
 	ret = PyObject_New(RemoteAccessObject, &RemoteAccess_Type);
@@ -739,13 +753,16 @@ static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
     }
 
 	if ((PyObject *)auth == Py_None) {
-		auth_baton = NULL;
 		ret->auth = NULL;
-	} else {
-		/* FIXME: check auth is an instance of Auth_Type */
+		svn_auth_open(&auth_baton, apr_array_make(ret->pool, 0, sizeof(svn_auth_provider_object_t *)), ret->pool);
+	} else if (PyObject_TypeCheck(auth, &Auth_Type)) {
 		Py_INCREF(auth);
 		ret->auth = auth;
 		auth_baton = ret->auth->auth_baton;
+	} else {
+		PyErr_SetString(PyExc_TypeError, "auth argument is not an Auth object");
+		Py_DECREF(ret);
+		return NULL;
 	}
 
 	if (!check_error(svn_ra_create_callbacks(&callbacks2, ret->pool))) {
@@ -759,6 +776,7 @@ static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 	callbacks2->progress_func = py_progress_func;
 	callbacks2->auth_baton = auth_baton;
 	callbacks2->open_tmp_file = py_open_tmp_file;
+	Py_INCREF(progress_cb);
 	ret->progress_func = progress_cb;
 	callbacks2->progress_baton = (void *)ret;
 #if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
@@ -770,8 +788,19 @@ static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 	Py_BEGIN_ALLOW_THREADS
+#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+	err = svn_ra_open3(&ret->ra, ret->url, uuid,
+			   callbacks2, ret, config_hash, ret->pool);
+#else
+	if (uuid != NULL) {
+		PyErr_SetString(PyExc_TypeError, 
+			"uuid argument not supported with svn 1.4");
+		Py_DECREF(ret);
+		return NULL;
+	}
 	err = svn_ra_open2(&ret->ra, ret->url,
 			   callbacks2, ret, config_hash, ret->pool);
+#endif
 	Py_END_ALLOW_THREADS
 	if (!check_error(err)) {
 		Py_DECREF(ret);
@@ -907,7 +936,8 @@ static PyObject *ra_get_log(PyObject *self, PyObject *args, PyObject *kwargs)
 	}
 
 	if (include_merged_revisions) {
-		PyErr_SetString(PyExc_NotImplementedError, "include_merged_revisions not supported in Subversion 1.4");
+		PyErr_SetString(PyExc_NotImplementedError, 
+			"include_merged_revisions not supported in Subversion 1.4");
 		apr_pool_destroy(temp_pool);
 		return NULL;
 	}
@@ -1130,7 +1160,8 @@ static PyObject *ra_replay(PyObject *self, PyObject *args)
 	temp_pool = Pool(NULL);
 	if (temp_pool == NULL)
 		return NULL;
-	Py_INCREF(update_editor);
+	/* Only INCREF here, py_editor takes care of the DECREF */
+	Py_INCREF(update_editor); 
 	RUN_RA_WITH_POOL(temp_pool, ra,
 					  svn_ra_replay(ra->ra, revision, low_water_mark,
 									send_deltas, &py_editor, update_editor, 
@@ -1214,7 +1245,8 @@ static PyObject *ra_replay_range(PyObject *self, PyObject *args)
 
 	Py_RETURN_NONE;
 #else
-	PyErr_SetString(PyExc_NotImplementedError, "svn_ra_replay not available with Subversion 1.4");
+	PyErr_SetString(PyExc_NotImplementedError, 
+		"svn_ra_replay not available with Subversion 1.4");
 	return NULL;
 #endif
 }
@@ -1369,6 +1401,48 @@ static PyObject *ra_change_rev_prop(PyObject *self, PyObject *args)
 	apr_pool_destroy(temp_pool);
 	Py_RETURN_NONE;
 }
+
+static PyObject *py_dirent(svn_dirent_t *dirent, int dirent_fields)
+{
+	PyObject *ret, *obj;
+	ret = PyDict_New();
+	if (dirent_fields & SVN_DIRENT_KIND) {
+		obj = PyInt_FromLong(dirent->kind);
+		PyDict_SetItemString(ret, "kind", obj);
+		Py_DECREF(obj);
+	}
+	if (dirent_fields & SVN_DIRENT_SIZE) {
+		obj = PyLong_FromLong(dirent->size);
+		PyDict_SetItemString(ret, "size", obj);
+		Py_DECREF(obj);
+	}
+	if (dirent_fields & SVN_DIRENT_HAS_PROPS) {
+		obj = PyBool_FromLong(dirent->has_props);
+		PyDict_SetItemString(ret, "has_props", obj);
+		Py_DECREF(obj);
+	}
+	if (dirent_fields & SVN_DIRENT_CREATED_REV) {
+		obj = PyLong_FromLong(dirent->created_rev);
+		PyDict_SetItemString(ret, "created_rev", obj);
+		Py_DECREF(obj);
+	}
+	if (dirent_fields & SVN_DIRENT_TIME) {
+		obj = PyLong_FromLong(dirent->time);
+		PyDict_SetItemString(ret, "time", obj);
+		Py_DECREF(obj);
+	}
+	if (dirent_fields & SVN_DIRENT_LAST_AUTHOR) {
+		if (dirent->last_author != NULL) {
+			obj = PyString_FromString(dirent->last_author);
+		} else {
+			obj = Py_None;
+			Py_INCREF(obj);
+		}
+		PyDict_SetItemString(ret, "last_author", obj);
+		Py_DECREF(obj);
+	}
+	return ret;
+}
 	
 static PyObject *ra_get_dir(PyObject *self, PyObject *args)
 {
@@ -1383,10 +1457,10 @@ static PyObject *ra_get_dir(PyObject *self, PyObject *args)
 	apr_ssize_t klen;
 	char *path;
 	svn_revnum_t revision = -1;
-	int dirent_fields = 0;
+	long dirent_fields = 0;
 	PyObject *py_dirents, *py_props;
 
-	if (!PyArg_ParseTuple(args, "s|li", &path, &revision, &dirent_fields))
+	if (!PyArg_ParseTuple(args, "s|lI", &path, &revision, &dirent_fields))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1400,36 +1474,24 @@ static PyObject *ra_get_dir(PyObject *self, PyObject *args)
 		fetch_rev = revision;
 
 	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_dir2(ra->ra, &dirents, &fetch_rev, &props,
-					 path, revision, dirent_fields, temp_pool));
+					 svn_path_canonicalize(path, temp_pool), revision, dirent_fields, temp_pool));
 
 	if (dirents == NULL) {
 		py_dirents = Py_None;
+		Py_INCREF(py_dirents);
 	} else {
 		py_dirents = PyDict_New();
 		idx = apr_hash_first(temp_pool, dirents);
 		while (idx != NULL) {
-			PyObject *py_dirent;
+			PyObject *item;
 			apr_hash_this(idx, (const void **)&key, &klen, (void **)&dirent);
-			py_dirent = PyDict_New();
-			if (dirent_fields & 0x1)
-				PyDict_SetItemString(py_dirent, "kind", 
-									 PyInt_FromLong(dirent->kind));
-			if (dirent_fields & 0x2)
-				PyDict_SetItemString(py_dirent, "size", 
-									 PyLong_FromLong(dirent->size));
-			if (dirent_fields & 0x4)
-				PyDict_SetItemString(py_dirent, "has_props",
-									 PyBool_FromLong(dirent->has_props));
-			if (dirent_fields & 0x8)
-				PyDict_SetItemString(py_dirent, "created_rev", 
-									 PyLong_FromLong(dirent->created_rev));
-			if (dirent_fields & 0x10)
-				PyDict_SetItemString(py_dirent, "time", 
-									 PyLong_FromLong(dirent->time));
-			if (dirent_fields & 0x20)
-				PyDict_SetItemString(py_dirent, "last_author",
-									 PyString_FromString(dirent->last_author));
-			PyDict_SetItemString(py_dirents, key, py_dirent);
+			item = py_dirent(dirent, dirent_fields);
+			if (item == NULL) {
+				Py_DECREF(py_dirents);
+				return NULL;
+			}
+			PyDict_SetItemString(py_dirents, key, item);
+			Py_DECREF(item);
 			idx = apr_hash_next(idx);
 		}
 	}
@@ -1466,7 +1528,7 @@ static PyObject *ra_get_file(PyObject *self, PyObject *args)
 	if (revision != SVN_INVALID_REVNUM)
 		fetch_rev = revision;
 
-	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_file(ra->ra, path, revision, 
+	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_file(ra->ra, svn_path_canonicalize(path, temp_pool), revision, 
 													new_py_stream(temp_pool, py_stream), 
 													&fetch_rev, &props, temp_pool));
 
@@ -1526,6 +1588,31 @@ static PyObject *ra_check_path(PyObject *self, PyObject *args)
 	return PyInt_FromLong(kind);
 }
 
+static PyObject *ra_stat(PyObject *self, PyObject *args)
+{
+	char *path; 
+	RemoteAccessObject *ra = (RemoteAccessObject *)self;
+	PyObject *ret;
+	svn_revnum_t revision;
+	svn_dirent_t *dirent;
+	apr_pool_t *temp_pool;
+
+	if (!PyArg_ParseTuple(args, "sl", &path, &revision))
+		return NULL;
+	if (ra_check_busy(ra))
+		return NULL;
+
+	temp_pool = Pool(NULL);
+	if (temp_pool == NULL)
+		return NULL;
+	RUN_RA_WITH_POOL(temp_pool, ra,
+					  svn_ra_stat(ra->ra, svn_path_canonicalize(path, temp_pool), revision, &dirent,
+					 temp_pool));
+	ret = py_dirent(dirent, SVN_DIRENT_ALL);
+	apr_pool_destroy(temp_pool);
+	return ret;
+}
+
 static PyObject *ra_has_capability(PyObject *self, PyObject *args)
 {
 #if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
@@ -1656,6 +1743,7 @@ static PyObject *ra_get_locks(PyObject *self, PyObject *args)
 			return NULL;
 		}
 		PyDict_SetItemString(ret, key, pyval);
+		Py_DECREF(pyval);
 	}
 
 	apr_pool_destroy(temp_pool);
@@ -1738,6 +1826,7 @@ static PyObject *mergeinfo_to_dict(svn_mergeinfo_t mergeinfo, apr_pool_t *temp_p
 		if (pyval == NULL)
 			return NULL;
 		PyDict_SetItemString(ret, key, pyval);
+		Py_DECREF(pyval);
 	}
 
 	return ret;
@@ -1791,6 +1880,7 @@ static PyObject *ra_mergeinfo(PyObject *self, PyObject *args)
 				return NULL;
 			}
 			PyDict_SetItemString(ret, key, pyval);
+			Py_DECREF(pyval);
 		}
 	}
 
@@ -1888,7 +1978,7 @@ static void ra_dealloc(PyObject *self)
 static PyObject *ra_repr(PyObject *self)
 {
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
-	return PyString_FromFormat("RemoteAccess(%s)", ra->url);
+	return PyString_FromFormat("RemoteAccess(\"%s\")", ra->url);
 }
 
 static int ra_set_progress_func(PyObject *self, PyObject *value, void *closure)
@@ -1906,37 +1996,89 @@ static PyGetSetDef ra_getsetters[] = {
 };
 
 static PyMethodDef ra_methods[] = {
-	{ "get_file_revs", ra_get_file_revs, METH_VARARGS, NULL },
-	{ "get_locations", ra_get_locations, METH_VARARGS, NULL },
-	{ "get_locks", ra_get_locks, METH_VARARGS, NULL },
-	{ "lock", ra_lock, METH_VARARGS, NULL },
-	{ "unlock", ra_unlock, METH_VARARGS, NULL },
-	{ "mergeinfo", ra_mergeinfo, METH_VARARGS, NULL },
-	{ "get_location_segments", ra_get_location_segments, METH_VARARGS, NULL },
-	{ "has_capability", ra_has_capability, METH_VARARGS, NULL },
-	{ "check_path", ra_check_path, METH_VARARGS, NULL },
-	{ "get_lock", ra_get_lock, METH_VARARGS, NULL },
-	{ "get_dir", ra_get_dir, METH_VARARGS, NULL },
-	{ "get_file", ra_get_file, METH_VARARGS, NULL },
-	{ "change_rev_prop", ra_change_rev_prop, METH_VARARGS, NULL },
-	{ "get_commit_editor", (PyCFunction)get_commit_editor, METH_VARARGS|METH_KEYWORDS, NULL },
-	{ "rev_proplist", ra_rev_proplist, METH_VARARGS, NULL },
-	{ "replay", ra_replay, METH_VARARGS, NULL },
-	{ "replay_range", ra_replay_range, METH_VARARGS, NULL },
-	{ "do_switch", ra_do_switch, METH_VARARGS, NULL },
-	{ "do_update", ra_do_update, METH_VARARGS, NULL },
-	{ "do_diff", ra_do_diff, METH_VARARGS, NULL },
-	{ "get_repos_root", (PyCFunction)ra_get_repos_root, METH_VARARGS|METH_NOARGS, NULL },
-	{ "get_log", (PyCFunction)ra_get_log, METH_VARARGS|METH_KEYWORDS, NULL },
-	{ "get_latest_revnum", (PyCFunction)ra_get_latest_revnum, METH_NOARGS, NULL },
-	{ "reparent", ra_reparent, METH_VARARGS, NULL },
-	{ "get_uuid", (PyCFunction)ra_get_uuid, METH_NOARGS, NULL },
+	{ "get_file_revs", ra_get_file_revs, METH_VARARGS, 
+		"S.get_file_revs(path, start_rev, end_revs, handler)" },
+	{ "get_locations", ra_get_locations, METH_VARARGS, 
+		"S.get_locations(path, peg_revision, location_revisions)" },
+	{ "get_locks", ra_get_locks, METH_VARARGS, 
+		"S.get_locks(path)" },
+	{ "lock", ra_lock, METH_VARARGS, 
+		"S.lock(path_revs, comment, steal_lock, lock_func)\n" },
+	{ "unlock", ra_unlock, METH_VARARGS, 
+		"S.unlock(path_tokens, break_lock, lock_func)\n" },
+	{ "mergeinfo", ra_mergeinfo, METH_VARARGS, 
+		"S.mergeinfo(&paths, revision, inherit, include_descendants)\n" },
+	{ "get_location_segments", ra_get_location_segments, METH_VARARGS, 
+		"S.get_location_segments(path, peg_revision, start_revision, "
+			"end_revision, rcvr)" 
+	},
+	{ "has_capability", ra_has_capability, METH_VARARGS, 
+		"S.has_capability(name) -> bool\n"
+		"Check whether the specified capability is supported by the client and server" },
+	{ "check_path", ra_check_path, METH_VARARGS, 
+		"S.check_path(path, revnum) -> node_kind\n"
+		"Check the type of a path (one of NODE_DIR, NODE_FILE, NODE_UNKNOWN)" },
+	{ "stat", ra_stat, METH_VARARGS,
+		"S.stat(path, revnum) -> dirent\n" },
+	{ "get_lock", ra_get_lock, METH_VARARGS, 
+		"S.get_lock(path) -> lock\n"
+	},
+	{ "get_dir", ra_get_dir, METH_VARARGS, 
+		"S.get_dir(path, revision, dirent_fields=-1) -> (dirents, fetched_rev, properties)\n"
+		"Get the contents of a directory. "},
+	{ "get_file", ra_get_file, METH_VARARGS, 
+		"S.get_file(path, stream, revnum=-1) -> (fetched_rev, properties)\n"
+		"Fetch a file. The contents will be written to stream." },
+	{ "change_rev_prop", ra_change_rev_prop, METH_VARARGS, 
+		"S.change_rev_prop(revnum, name, value)\n"
+		"Change a revision property" },
+	{ "get_commit_editor", (PyCFunction)get_commit_editor, METH_VARARGS|METH_KEYWORDS, 
+		"S.get_commit_editor(revprops, commit_callback, lock_tokens, keep_locks) -> editor\n"
+	},
+	{ "rev_proplist", ra_rev_proplist, METH_VARARGS, 
+		"S.rev_proplist(revnum) -> properties\n"
+		"Return a dictionary with the properties set on the specified revision" },
+	{ "replay", ra_replay, METH_VARARGS, 
+		"S.replay(revision, low_water_mark, update_editor, send_deltas=True)\n" 
+		"Replay a revision, reporting changes to update_editor." },
+	{ "replay_range", ra_replay_range, METH_VARARGS, 
+		"S.replay_range(start_rev, end_rev, low_water_mark, cbs, send_deltas=True)\n"
+		"Replay a range of revisions, reporting them to an update editor.\n"
+		"cbs is a two-tuple with two callbacks:\n"
+		"  start_rev_cb(revision, revprops) -> editor\n"
+		"  finish_rev_cb(revision, revprops, editor)\n"
+	},
+	{ "do_switch", ra_do_switch, METH_VARARGS, 
+		"S.do_switch(revision_to_update_to, update_target, recurse, switch_url, update_editor)\n" },
+	{ "do_update", ra_do_update, METH_VARARGS, 
+		"S.do_update(revision_to_update_to, update_target, recurse, update_editor)\n" },
+	{ "do_diff", ra_do_diff, METH_VARARGS, 
+		"S.do_diff(revision_to_update_to, diff_target, versus_url, diff_editor, recurse, ignore_ancestry, text_deltas)\n"
+	},
+	{ "get_repos_root", (PyCFunction)ra_get_repos_root, METH_VARARGS|METH_NOARGS, 
+		"S.get_repos_root() -> url\n"
+		"Return the URL to the root of the repository." },
+	{ "get_log", (PyCFunction)ra_get_log, METH_VARARGS|METH_KEYWORDS, 
+		"S.get_log(paths, start, end, limit, discover_changed_paths, "
+		"strict_node_history, include_merged_revisions, revprops)\n"
+	},
+	{ "get_latest_revnum", (PyCFunction)ra_get_latest_revnum, METH_NOARGS, 
+		"S.get_latest_revnum() -> int\n"
+		"Return the last revision committed in the repository." },
+	{ "reparent", ra_reparent, METH_VARARGS, 
+		"S.reparent(url)\n"
+		"Reparent to a new URL" },
+	{ "get_uuid", (PyCFunction)ra_get_uuid, METH_NOARGS, 
+		"S.get_uuid() -> uuid\n"
+		"Return the UUID of the repository." },
 	{ NULL, }
 };
 
 static PyMemberDef ra_members[] = {
-	{ "busy", T_BYTE, offsetof(RemoteAccessObject, busy), READONLY, NULL },
-	{ "url", T_STRING, offsetof(RemoteAccessObject, url), READONLY, NULL },
+	{ "busy", T_BYTE, offsetof(RemoteAccessObject, busy), READONLY, 
+		"Whether this connection is in use at the moment" },
+	{ "url", T_STRING, offsetof(RemoteAccessObject, url), READONLY, 
+		"URL this connection is to" },
 	{ NULL, }
 };
 
@@ -2054,13 +2196,15 @@ static PyObject *auth_init(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 
 	if (!PyList_Check(providers)) {
 		PyErr_SetString(PyExc_TypeError, "Auth providers should be list");
-		Py_DECREF(ret);
+		PyObject_Del(ret);
 		return NULL;
 	}
 
 	ret->pool = Pool(NULL);
 	if (ret->pool == NULL) {
-		Py_DECREF(ret);
+		PyErr_NoMemory();
+		apr_pool_destroy(ret->pool);
+		PyObject_Del(ret);
 		return NULL;
 	}
 
@@ -2070,7 +2214,8 @@ static PyObject *auth_init(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 	c_providers = apr_array_make(ret->pool, PyList_Size(providers), sizeof(svn_auth_provider_object_t *));
 	if (c_providers == NULL) {
 		PyErr_NoMemory();
-		Py_DECREF(ret);
+		apr_pool_destroy(ret->pool);
+		PyObject_Del(ret);
 		return NULL;
 	}
 	for (i = 0; i < PyList_Size(providers); i++) {
@@ -2078,6 +2223,12 @@ static PyObject *auth_init(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 		el = (svn_auth_provider_object_t **)apr_array_push(c_providers);
 		/* FIXME: Check that provider is indeed a AuthProviderObject object */
 		provider = (AuthProviderObject *)PyList_GetItem(providers, i);
+		if (!PyObject_TypeCheck(provider, &AuthProvider_Type)) {
+			PyErr_SetString(PyExc_TypeError, "Invalid auth provider");
+			apr_pool_destroy(ret->pool);
+			PyObject_Del(ret);
+			return NULL;
+		}
 		*el = provider->provider;
 	}
 	svn_auth_open(&ret->auth_baton, c_providers, ret->pool);
@@ -2272,9 +2423,14 @@ PyTypeObject CredentialsIter_Type = {
 };
 
 static PyMethodDef auth_methods[] = {
-	{ "set_parameter", auth_set_parameter, METH_VARARGS, NULL },
-	{ "get_parameter", auth_get_parameter, METH_VARARGS, NULL },
-	{ "credentials", auth_first_credentials, METH_VARARGS, NULL },
+	{ "set_parameter", auth_set_parameter, METH_VARARGS, 
+		"S.set_parameter(key, value)\n"
+		"Set a parameter" },
+	{ "get_parameter", auth_get_parameter, METH_VARARGS, 
+		"S.get_parameter(key) -> value\n"
+		"Get a parameter" },
+	{ "credentials", auth_first_credentials, METH_VARARGS, 
+		"Credentials" },
 	{ NULL, }
 };
 
@@ -2499,6 +2655,7 @@ static svn_error_t *py_ssl_server_trust_prompt(svn_auth_cred_ssl_server_trust_t
 
 	if (cert_info == NULL) {
 		py_cert = Py_None;
+		Py_INCREF(py_cert);
 	} else {
 		py_cert = Py_BuildValue("(sssss)", cert_info->hostname, cert_info->fingerprint, 
 						  cert_info->valid_from, cert_info->valid_until, 
@@ -2798,7 +2955,9 @@ static PyObject *get_keychain_simple_provider(PyObject* self)
 #endif
 
 static PyMethodDef ra_module_methods[] = {
-	{ "version", (PyCFunction)version, METH_NOARGS, NULL },
+	{ "version", (PyCFunction)version, METH_NOARGS, 
+		"version() -> (major, minor, micro, tag)\n"
+		"Return version string of ra library."},
 	{ "get_ssl_client_cert_pw_file_provider", (PyCFunction)get_ssl_client_cert_pw_file_provider, METH_NOARGS, NULL },
 	{ "get_ssl_client_cert_file_provider", (PyCFunction)get_ssl_client_cert_file_provider, METH_NOARGS, NULL },
 	{ "get_ssl_server_trust_file_provider", (PyCFunction)get_ssl_server_trust_file_provider, METH_NOARGS, NULL },
