@@ -1,35 +1,49 @@
 # Copyright (C) 2005-2007 Jelmer Vernooij <jelmer@samba.org>
  
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation; either version 2.1 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Lesser General Public License for more details.
 
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Subversion ra library tests."""
 
 from cStringIO import StringIO
 from unittest import TestCase
-from subvertpy import ra, SubversionException, NODE_DIR, NODE_NONE
+
+from subvertpy import (
+    NODE_DIR, NODE_NONE,
+    SubversionException,
+    ra,
+    )
 from subvertpy.tests import SubversionTestCase
 
 class VersionTest(TestCase):
+
     def test_version_length(self):
         self.assertEquals(4, len(ra.version()))
 
 
+class TestRemoteAccessUnknown(TestCase):
+
+    def test_unknown_url(self):
+        self.assertRaises(SubversionException, ra.RemoteAccess, "bla://")
+
+
 class TestRemoteAccess(SubversionTestCase):
+
     def setUp(self):
         super(TestRemoteAccess, self).setUp()
         self.repos_url = self.make_repository("d")
-        self.ra = ra.RemoteAccess(self.repos_url)
+        self.ra = ra.RemoteAccess(self.repos_url,
+                auth=ra.Auth([ra.get_username_provider()]))
 
     def commit_editor(self):
         return self.get_commit_editor(self.repos_url)
@@ -40,7 +54,7 @@ class TestRemoteAccess(SubversionTestCase):
         dc.close()
 
     def test_repr(self):
-        self.assertEquals("RemoteAccess(%s)" % self.repos_url,
+        self.assertEquals("RemoteAccess(\"%s\")" % self.repos_url,
                           repr(self.ra))
 
     def test_latest_revnum(self):
@@ -105,11 +119,18 @@ class TestRemoteAccess(SubversionTestCase):
             returned.append(args)
         def check_results(returned):
             self.assertEquals(2, len(returned))
-            (paths, revnum, props, has_children) = returned[0]
+            self.assert_(len(returned[0]) in (3,4))
+            if len(returned[0]) == 3:
+                (paths, revnum, props) = returned[0]
+            else:
+                (paths, revnum, props, has_children) = returned[0]
             self.assertEquals(None, paths)
             self.assertEquals(revnum, 0)
             self.assertEquals(["svn:date"], props.keys())
-            (paths, revnum, props, has_children) = returned[1]
+            if len(returned[1]) == 3:
+                (paths, revnum, props) = returned[1]
+            else:
+                (paths, revnum, props, has_children) = returned[1]
             self.assertEquals({'/foo': ('A', None, -1)}, paths)
             self.assertEquals(revnum, 1)
             self.assertEquals(set(["svn:date", "svn:author", "svn:log"]), 
@@ -219,6 +240,14 @@ class TestRemoteAccess(SubversionTestCase):
         self.assertEquals(NODE_DIR, self.ra.check_path("bar/", 1))
         self.assertEquals(NODE_NONE, self.ra.check_path("blaaaa", 1))
 
+    def test_stat(self):
+        cb = self.commit_editor()
+        cb.add_dir("bar")
+        cb.close()
+
+        ret = self.ra.stat("bar", 1)
+        self.assertEquals(set(['last_author', 'kind', 'created_rev', 'has_props', 'time', 'size']), set(ret.keys()))
+
     def test_get_locations_dir(self):
         cb = self.commit_editor()
         cb.add_dir("bar")
@@ -245,6 +274,10 @@ class TestRemoteAccess(SubversionTestCase):
                           self.ra.get_locations("bla", 3, [1,2,3]))
 
 class AuthTests(TestCase):
+
+    def test_not_list(self):
+        self.assertRaises(TypeError, ra.Auth, ra.get_simple_provider())
+
     def test_not_registered(self):
         auth = ra.Auth([])
         self.assertRaises(SubversionException, auth.credentials, "svn.simple", "MyRealm")
